#include "DemoMainFrame.h"

// Diese Makros steuern fr das jeweilige Window-Objekt,
// wie Messages in Funktionsaufrufe umgesetzt werden.
BEGIN_MESSAGE_MAP(DemoMainFrame, CFrameWnd)
	// Windows Messages:
	ON_WM_PAINT()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_SIZE()
	// Notification Messages (etwas ausfhrlicher anzugeben):
	ON_BN_CLICKED (100, OnClickClear)
	// 100 = Kennzahl fr Button als Unterfenster
	// Kennzahlen unter 100 sind reserviert.
END_MESSAGE_MAP()

DemoMainFrame::DemoMainFrame() :
myPaintCounter(1),
myDotCounter(0),
myPoint(0,0)
{
	Create(NULL, "Test", WS_OVERLAPPEDWINDOW, CRect(100, 100, 400, 400));
	// Bedeutung der Parameter:
	// Fensterklasse, Titel, verstellbares Fenster mit den blichen Elementen, Anfangsposition

	// mit bitweisem Oder: WS_OVERLAPPEDWINDOW =
	// WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WX_MAXIMIZEBOX

	// NULL ist historisch. Standard-C++: einfach 0 schreiben

	myStatusBar.Create(WS_CHILD | WS_VISIBLE, CRect(1, 2, 3, 4), this, 101);
	// Das CRect hier wird ignoriert; Position und Gre der Statusleiste
	// folgen logischerweise dem Mutterfenster. 
	// 101 ist eine Kennzahl fr die Statusleiste als Unterfenster
	int statusParts[] = {150, 200, 250};
	// Endkoordinaten der Abteilungen auf der Statusleiste
	myStatusBar.SetParts(3, statusParts);
	// drei Abteilungen vorgegebener Lnge auf der Statusleiste

	myButton.Create ("Lschen", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
							 CRect(CPoint(40, 90), CSize(100, 20)), this, 100);
}

void DemoMainFrame::OnPaint() // wird fr MW_PAINT-Messages aufgerufen: Neuzeichnen
{
	CPaintDC dc(this); // Grafikkontext holen und MW_PAINT-Messages lschen
	
	// Die folgenden Zeilen sind nur ntig,
	// damit wir nicht ber die Statusleiste malen.
	// Das Problem erledigt sich, wenn man Unterfenster
	// (in Swing genannt: Komponenten) benutzt,
	// statt direkt ins Fenster zu malen.
	// Am Button sieht man den Fehler noch.
	
	CRect r;
	myStatusBar.GetWindowRect(r);
	// Umriss der Statusleiste in absoluten Bildschirmkoordinaten
	// Ergebnis steht in r! (bergabe als C++-Referenz)
	ScreenToClient(r);
	// in lokale Koordinaten der DemoMainFrame-Clientflche umrechnen
	dc.ExcludeClipRect(r);
	// aus dem Clip-Rechteck ausnehmen, sodass nicht
	// darber gezeichnet wird



	dc.TextOut(40, 40, "Hallo Welt!");

	CString myString; // MFC hat eigene String-Klasse ...
	// ... mit der Typ-unsicheren Formatierung nach Art von C
	myString.Format("Zahl der Paint-Aufrufe: %d", myPaintCounter);
	++myPaintCounter;
	dc.TextOut(40, 60, myString);
	myStatusBar.SetText(myString, 0 /*erste Flche*/, 0 /*versunken*/);

	dc.MoveTo(myPoint.x, myPoint.y-50);
	dc.LineTo(myPoint.x, myPoint.y+50);
	dc.MoveTo(myPoint.x-50, myPoint.y);
	dc.LineTo(myPoint.x+50, myPoint.y);

	CPen myRedPen(PS_SOLID, 5, RGB(255,0,0));
	CBrush myDarkBrush(RGB(80, 120, 160));
	// RGB ist brigens ein Makro, kein Konstruktor

	CPen* oldPen = dc.SelectObject(&myRedPen); // fr Linien und Umrisse
	CBrush* oldBrush = dc.SelectObject(&myDarkBrush); // fr Fllungen
	// SelectObject liefert einen Zeiger auf
	// das bisherige CPen/CBrush zurck.
	// Brauchen wir unten.

	int i; // for(int i = 0; ...) ist ja bei MS nicht standardkonform gelst.
	for(i=0; i<myDotCounter; i++)
	{
		dc.Ellipse(myDots[i].x-10, myDots[i].y-10, myDots[i].x+10, myDots[i].y+10);
	}

	dc.SelectObject(oldPen);
	dc.SelectObject(oldBrush);
	// zurckstellen, sonst knnen myRedPen und myDarkBrush
	// beim Verlassen dieser Methode nicht ordentlich
	// zerstrt werden. Nach einigen hundert Aufrufen
	// dieser Methode kommt es dann wegen Speichermangel
	// zu Grafikfehlern.
}

void DemoMainFrame::OnMouseMove(UINT nFlags, CPoint point)
// UINT: Microsoftsch fr unsigned int. Von der Sorte gibts noch mehr.
{
	myPoint = point;
	Invalidate(TRUE); // wie repaint bei Java; TRUE steht fr "vorher lschen" (Standard-C++: true)
	// Der Bildschirm flackert: Kein Double Buffering wie bei Java!
	CString myString;
	myString.Format("x: %d", point.x);
	myStatusBar.SetText(myString, 1 /*zweite Flche*/, SBT_NOBORDERS /*ohne Rahmen*/);
	myString.Format("y: %d", point.y);
	myStatusBar.SetText(myString, 2 /*dritte Flche*/, SBT_POPOUT /*ragt heraus*/);
}

void DemoMainFrame::OnLButtonDown(UINT nFlags, CPoint point)
{
	if(myDotCounter > 9)
	{
		MessageBox("Mehr geht nicht!", "Dumme Meldung", MB_ICONEXCLAMATION);
		// andere Mglichkeiten z.B.
		// MB_ICONERROR, MB_ICONQUESTION, MB_ICONINFORMATION
		// MessageBox ist eine Methode des Window-Objekts
		return;
	}
	myDots[myDotCounter] = point;
	++myDotCounter;
	Invalidate(TRUE);
}

void DemoMainFrame::OnSize(UINT nType, int cx, int cy)
// nur, um Statuszeile bei Grennderungen am unteren Fensterrand zu halten
{	
	// Die Statuszeile muss erfahren, dass die Fenstergre
	// gendert wurde, damit sie sich an die neue Position zeichnet.
	
	// Mglichkeit 1:
	// myStatusBar.MoveWindow(1, 2, 3, 4); // Koordinaten hier egal

	// Mglichkeit 2:	
	SendMessageToDescendants(WM_SIZE);
}

void DemoMainFrame::OnClickClear()
{
	myDotCounter = 0;
	Invalidate(TRUE);
}